class TreeviewNode {
  constructor(treeview, node) {
    this.Treeview = treeview;
    this.Node = node;
  }


  get Expandbox() {
    const a = this;

    const result = a.Node.querySelectorAll("div.icon.expand");
    if (typeof(result) == "undefined") {
      return null;
    }

    return result[0];
  }

  get Checkbox() {
    const a = this;

    if (!a.Treeview.Options.ShowCheckbox) {
      return null;
    }

    const result = a.Node.querySelectorAll("div.icon.checkbox");
    if (typeof(result) == "undefined") {
      return null;
    }

    return result[0];
  }

  get ChildNode() {
    const a = this;

    let result = a.Node.querySelectorAll("ul");
    if (result.length <= 0) {
      return null;
    }

    return result[0];
  }

  get ChildNodes() {
    const a = this;

    const childNode = a.ChildNode;
    if (childNode == null) {
      return [];
    }

    let nodes = childNode.querySelectorAll("li");
    if (nodes.length <= 0) {
      return [];
    }

    let result = [];
    nodes.forEach(function(e) {
      if (typeof(e.parentNode) == "undefined") {
        return;
      }

      if (e.parentNode != childNode) {
        return;
      }

      result.push(e);
    });

    return result;
  }

  get Container() {
    return this.Node.querySelectorAll("div.li")[0];
  }

  get Hint() {
    return this.Label.getAttribute("title");
  }

  get ID() {
    return this.Node.getAttribute("data-bbtv-id");
  }


  get IsChecked() {
    const a = this;

    if (!a.Treeview.Options.ShowCheckbox) {
      return false;
    }

    return a.Node.classList.contains("x");
  }

  get IsExpanded() {
    return this.Node.classList.contains("e");
  }

  get IsHighlighted() {
    return this.Container.classList.contains("highlighted");
  }

  get Label() {
    return this.Node.querySelectorAll("span")[0];
  }

  get Name() {
    return this.Label.textContent;
  }

  get ParentID() {
    const a = this;

    const parentNode = a.ParentNode;
    if (parentNode == null) {
      return null;
    }

    return parentNode.getAttribute("data-bbtv-id");
  }

  get ParentNode() {
    const a = this;

    return a.#parentsUntilTagName(a.Node, "li");
  }

  get Tag() {
    const a = this;

    let tag = a.Node.getAttribute("data-bbtv-tag");
    if (a.Treeview.isNullOrWhitespace(tag)) {
      return null;
    }

    return JSON.parse(decodeURIComponent(tag));
  }

  get Value() {
    return this.Node.getAttribute("data-bbtv-value");
  }


  Check(value) {
    const a = this;

    a.#setCheckbox(a.Node, value);

    // Update children
    a.Node.querySelectorAll("li").forEach(function(e) {
      a.#setCheckbox(e, value);
    });

    let parentNode = a.ParentNode;
    if (parentNode == null) {
      return;
    }

    // Handle pull-ups
    if (a.Treeview.Options.EnableGuardian) {
      while (true) {
        const parentChecked = (parentNode.querySelectorAll("li.x").length > 0);

        a.#setCheckbox(parentNode, parentChecked);

        parentNode = a.Treeview.GetTreeNode(parentNode);
        parentNode = parentNode.ParentNode;

        if (parentNode == null) {
          break;
        }
      }
    }

    if (a.Treeview.Options.EnableCascade) {
      let uncheckedCount = 0;

      parentNode.querySelectorAll("li").forEach(function(e) {
        if (e.classList.contains("x")) {
          return;
        }

        uncheckedCount++;
      });

      a.#setCheckbox(parentNode, (uncheckedCount <= 0));
    }

  }

  Expand(value) {
    const a = this;

    a.#setExpandbox(a.Node, value);
  }

  Remove() {
    const a = this;

    if (a.ParentNode != null) {
      if (a.ParentNode.classList.contains("e")) {
        a.ParentNode.classList.remove("e");
      }

      if (a.ParentNode.classList.contains("c")) {
        a.ParentNode.classList.remove("c");
      }
    }

    a.Node.parentNode.removeChild(a.Node);
  }

  InitialiseComponents() {
    const a = this;

    // fold
    a.#initialiseComponents_Expand();

    // checkbox behaviour
    a.#initialiseComponents_Checkbox();

    // highlighting + collapsible
    a.#initialiseComponents_Container();

    a.Expand(a.IsExpanded);
    a.Check(a.IsChecked);
  }


  #initialiseComponents_Expand() {
    const a = this;

    a.Expandbox.addEventListener("mousedown", function(e){
      e.stopPropagation();
      e.preventDefault();

      a.Expand(!a.IsExpanded);
    });

    a.Expandbox.addEventListener("click", function(e){
      e.stopPropagation();
      e.preventDefault();

      // do nothing
    });

    a.Expandbox.addEventListener("dblclick", function(e){
      e.stopPropagation();
      e.preventDefault();

      // do nothing
    });
  }

  #initialiseComponents_Checkbox() {
    const a = this;

    if (!a.Treeview.Options.ShowCheckbox) {
      return;
    }

    a.Checkbox.addEventListener("mousedown", function(e){
      e.stopPropagation();
      e.preventDefault();

      a.Check(!a.IsChecked);
    });

    a.Checkbox.addEventListener("click", function(e){
      e.stopPropagation();
      e.preventDefault();

      // do nothing
    });

    a.Checkbox.addEventListener("dblclick", function(e){
      e.stopPropagation();
      e.preventDefault();

      // do nothing
    });
  }

  #initialiseComponents_Container() {
    const a = this;

    // collapsible
    a.Container.addEventListener("dblclick", function(e){
      e.stopPropagation();
      e.preventDefault();

      const myNode = a.Treeview.GetTreeNode(e.target);
      if (myNode == null) {
        return;
      }

      a.Expand(!a.IsExpanded);
    });

    // highlighting
    a.Container.addEventListener("mousedown", function(e){
      e.stopPropagation();
      e.preventDefault();

      const myNode = a.Treeview.GetTreeNode(e.target);
      if (myNode == null) {
        return;
      }

      a.Treeview.Container.querySelectorAll("div.highlighted").forEach(function(e) {
        e.classList.remove("highlighted");
      });

      // Don't show selection
      if (!a.Treeview.Options.ShowSelection) {
        return;
      }

      myNode.Container.classList.add("highlighted");
    });
  }

  #isTag(el, tagName) {
    return (el.tagName.toLowerCase() == tagName);
  }

  #parentsUntilTagName(el, tagName) {
    const a = this;

    let node = el;

    while (true) {
      if (typeof(node.parentNode) == "undefined") {
        break;
      }

      node = node.parentNode;

      if (typeof(node) == "undefined") {
        break;
      }

      if (a.#isTag(node, "ul")) {
        if (node.classList.contains("bbtreeview")) {
          return null;
        }
      }

      if (a.#isTag(node, tagName)) {
        break;
      }
    }

    return node;
  }

  #setCheckbox(el, value) {
    el.classList.remove("x");
    el.classList.remove("o");

    el.classList.add((value ? "x" : "o"));
  }

  #setExpandbox(el, value) {
    el.classList.remove("e");
    el.classList.remove("c");

    if (value) {
      if (el.querySelectorAll("li").length > 0) {
        el.classList.add("e");
      } else {
        el.classList.add("nc");
      }
    } else {
      el.classList.add("c");
    }
  }

}


export default TreeviewNode;